@@ -1,10 +1,3 @@
-"""
-.. module:: MalConTestSuite
-	:synopsis: MalConv basic unit tests
-
-.. moduleauthor:: Luca Demetrio <luca.demetrio@dibris.unige.it>
-"""
-
 import unittest
 import torch
 from secml.array import CArray
